package com.github.confluencecamera;

import org.xmlrpc.android.XMLRPCException;

import android.app.Activity;
import android.content.Context;
import android.content.DialogInterface;
import android.content.SharedPreferences;
import android.hardware.Camera;
import android.hardware.Camera.CameraInfo;
import android.hardware.Camera.PictureCallback;
import android.hardware.Camera.ShutterCallback;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.FrameLayout;

/**
 * This is app's main activity.
 * @author mvbrocato
 */
public class CameraActivity extends Activity {
	private static final String PREFS_FILE = "settings";
	private static final String TAG = "WhiteboardCapture";
	
	private ConfluenceProxy proxy;
	private SharedPreferences prefs;
	private Preview preview;
	private Camera camera;
	private int defaultCameraId;
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);
		loadPrefs();
		initConfluenceProxy();
		//initPageField();
		initPreview();
	}

	/**
	 * Grabs control of the camera and assigns it to the preview.
	 */
	protected void onResume() {
		super.onResume();
		camera = Camera.open(defaultCameraId);
		preview.setCamera(camera);
	}

	/**
	 * Releases control of the camera
	 */
    protected void onPause() {
        super.onPause();

        // Because the Camera object is a shared resource, it's very
        // important to release it when the activity is paused.
        if (camera != null) {
            preview.setCamera(null);
            camera.release();
            camera = null;
        }
    }
	
    /**
     * Builds the camera preview components
     */
	private void initPreview() {
        int numberOfCameras = Camera.getNumberOfCameras();
        
        // Find the ID of the default camera
        CameraInfo cameraInfo = new CameraInfo();
        for (int i = 0; i < numberOfCameras; i++) {
            Camera.getCameraInfo(i, cameraInfo);
            if (cameraInfo.facing == CameraInfo.CAMERA_FACING_BACK) {
                defaultCameraId = i;
            }
        }
		
		preview = new Preview(this);
		
		((FrameLayout) findViewById(R.id.preview)).addView(preview);

		Button captureButton = (Button) findViewById(R.id.captureButton);
		captureButton.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				camera.takePicture(shutterCallback, rawCallback, jpegCallback);
			}
		});
	}

	/**
	 * Loads user preferences from storage
	 */
	private void loadPrefs() {
		prefs = getSharedPreferences(PREFS_FILE, Context.MODE_PRIVATE);
	}

	/**
	 * Initializes the confluence XML-RPC proxy.  If the proxy fails to connect, the settings
	 * dialog is shown.
	 */
	private void initConfluenceProxy() {
		proxy = new ConfluenceProxy(prefs.getString("url", ""), prefs.getString("username", ""), prefs.getString("password", ""));

		try {
			proxy.login();
		} catch(XMLRPCException e) {
			// allow user to fix settings if login fails
			// TODO improve this
			this.showSettings();
		}
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		super.onCreateOptionsMenu(menu);
		Log.d(TAG, "onCreateOptionsMenu called");
		MenuInflater i = getMenuInflater();
		i.inflate(R.menu.main, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case R.id.settings:
			showSettings();
			return true;
		default:
			return super.onOptionsItemSelected(item);
		}
	}

	ShutterCallback shutterCallback = new ShutterCallback() {
		public void onShutter() {
			Log.d(TAG, "onShutter'd");
		}
	};

	/** 
	 * Handles data for raw picture 
	 */
	PictureCallback rawCallback = new PictureCallback() {
		public void onPictureTaken(byte[] data, Camera camera) {
			Log.d(TAG, "onPictureTaken - raw");
		}
	};

	/**
	 * Handles data for jpeg picture 
	 */
	PictureCallback jpegCallback = new PictureCallback() {
		public void onPictureTaken(byte[] data, final Camera camera) {
			UploadDialog d = new UploadDialog(CameraActivity.this, data, proxy);

			// start previewing again when the dialog is dismissed;
			d.setOnDismissListener(new UploadDialog.OnDismissListener() {
				@Override
				public void onDismiss(DialogInterface dialog) {
					camera.startPreview();
				}
			});
			
			d.show();
			Log.d(TAG, "onPictureTaken - wrote bytes: " + data.length);
		}
	};

	/**
	 * Opens the settings dialog
	 */
	private void showSettings() {
		new SettingsDialog(this, prefs).show();
	}
}