package com.github.confluencecamera;

import java.util.LinkedList;
import java.util.List;

import org.xmlrpc.android.XMLRPCException;

import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.AsyncTask;
import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.view.WindowManager;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.Toast;

import com.github.confluencecamera.model.Page;

/**
 * This dialog allows the user to see the picture they've taken, select a
 * confluence page, and finally upload the picture.
 * 
 * @author mvbrocato
 */
public class UploadDialog extends Dialog {
	private static final String TAG = "UploadDialog";
	
	private ListView pageList;
	private EditText pageQuery;
	private ArrayAdapter<String> pageListAdapter;
	private ConfluenceProxy proxy;
	private AsyncTask<String, Void, List<Page>> searchTask; 
	private ProgressDialog progressDialog;
	
	public UploadDialog(final Context context, final byte[] image, final ConfluenceProxy proxy) {
		super(context);
		setContentView(R.layout.upload);
		this.proxy = proxy;
		
		initLayout();
		initPageList();
		
		this.setTitle("Upload Image");

		// convert image byte array to bmp
		Bitmap bmp = BitmapFactory.decodeByteArray(image, 0, image.length);
		ImageView imageView = new ImageView(context);
		imageView.setImageBitmap(bmp);

		// display the image inside the placeholder FrameLayout
		FrameLayout imageFrame = (FrameLayout) findViewById(R.id.uploadImage);
		imageFrame.addView(imageView);

		Button cancelButton = (Button) findViewById(R.id.cancelUploadButton);
		cancelButton.setOnClickListener(new Button.OnClickListener() {
			public void onClick(View v) {
				UploadDialog.this.dismiss();
			}
		});

		Button uploadButton = (Button) findViewById(R.id.uploadButton);
		uploadButton.setOnClickListener(new Button.OnClickListener() {
			public void onClick(View v) {
				try {
					proxy.uploadImage(image);
				} catch (XMLRPCException e) {
					Log.e(TAG, "Error uploading image.", e);
				}
				Toast.makeText(context, "Image Uploaded to Confluence", Toast.LENGTH_SHORT).show();
				UploadDialog.this.dismiss();
			}
		});
	}

	private void initPageList() {
		progressDialog = new ProgressDialog(getContext());
		progressDialog.setMessage("Loading");
		
		pageQuery = (EditText) findViewById(R.id.pageQuery);
		pageQuery.setOnKeyListener(new EditText.OnKeyListener() {
			@Override
			public boolean onKey(View v, int keyCode, KeyEvent event) {
				if(event.getAction() == KeyEvent.ACTION_UP) {
					updatePageList(pageQuery.getText().toString());
				}
				return false;
			}
		});
		
		pageList = (ListView) findViewById(R.id.pageList);
		pageListAdapter = new ArrayAdapter<String>(this.getContext(), android.R.layout.simple_dropdown_item_1line, new LinkedList<String>());
		pageList.setAdapter(pageListAdapter);
		
		updatePageList(null);
	}

	/**
	 * Refreshes the items in the page list to match the query typed in by the user.
	 * @param query
	 */
	private void updatePageList(final String query) {
		if(searchTask != null) {
			/*
			 * If a search task was kicked off from another keystroke, cancel it and 
			 * start a new task.
			 */
			searchTask.cancel(true);
		}
		
		searchTask = new AsyncTask<String, Void, List<Page>>() {
			protected List<Page> doInBackground(String... params) {
				try {
					/*
					 * Wait for the user to stop typing. 
					 */
					Thread.sleep(500);
					
					/*
					 * Showing a dialog creates a handler, and since handlers can only be shown on UI threads
					 * and doInBackground occurs in a separate thread, we have to show the dialog in onProgressUpdate 
					 * instead.
					 */
					this.publishProgress();
					
					return proxy.searchPages(params[0] + "*", 10);
				} catch (InterruptedException e) {
					return null;
				} catch (XMLRPCException e) {
					Log.e(TAG, "Error fetching matching pages", e);
					return new LinkedList<Page>();
				}
			}

			protected void onProgressUpdate(Void... values) {
				progressDialog.show();
			}
			
			@Override
			protected void onPostExecute(List<Page> pages) {
				if (pages != null) {
					pageListAdapter.clear();
					for (Page page: pages) {
						Log.d(TAG, page.getTitle());
						pageListAdapter.add(page.getTitle());
					}
					pageListAdapter.notifyDataSetChanged();
					progressDialog.hide();
				}
			}
		};

		searchTask.execute(query);
	}

	/**
	 * Make this dialog full screen.
	 */
	private void initLayout() {
		WindowManager.LayoutParams lp = new WindowManager.LayoutParams();
		lp.copyFrom(getWindow().getAttributes());
		lp.width = WindowManager.LayoutParams.FILL_PARENT;
		lp.height = WindowManager.LayoutParams.FILL_PARENT;
		show();
		getWindow().setAttributes(lp);
	}
}
